#include <libwccl/ops/actions/delete.h>
#include <libwccl/ops/functions/constant.h>
#include <libpwrutils/foreach.h>
#include <sstream>

namespace Wccl {

Bool Delete::execute(const ActionExecContext& context) const
{
	Bool changed(false);
	const boost::shared_ptr<const Position> pos = pos_->apply(context);
	int abs_pos = context.sentence_context().get_abs_position(*pos);
	if (context.sentence_context().is_inside(abs_pos)) {
		Corpus2::Token& token = *context.sentence_context().at(abs_pos);
		std::vector<Corpus2::Lexeme> original(token.lexemes());
		std::vector<Corpus2::Lexeme> remaining;
		token.lexemes() = remaining;
		foreach (const Corpus2::Lexeme& lexeme, original) {
			token.add_lexeme(lexeme);
			if (!condition_->apply(context)->get_value()) {
				remaining.push_back(lexeme);
			}
			token.lexemes().pop_back();
		}
		changed.set_value((remaining.size() > 0) && (remaining.size() < original.size()));
		token.lexemes() = changed.get_value() ? remaining : original;
	}
	return changed;
}

std::string Delete::to_string(const Corpus2::Tagset& tagset) const
{
	std::ostringstream os;
	os << name() << "(" << pos_->to_string(tagset) << ", "
			<< condition_->to_string(tagset) << ")";
	return os.str();
}

std::ostream& Delete::write_to(std::ostream& os) const
{
	os << name() << "(" << *pos_ << ", " << *condition_ << ")";
	return os;
}


} /* end ns Wccl */
